--[[
	QuickTargetMacros
    Copyright (C) 2010  Morsker of Blackhand-US
	Please contact Morsker through PM on forums.wowace.com.

	Aids the user in quickly making /target macros for immediate use.

	License:
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program. If not, see <http://www.gnu.org/licenses/>.

	Note:
		This AddOn's source code is specifically designed to work with
		World of Warcraft's interpreted AddOn system.
		You have an implicit license to use this AddOn with these facilities
		since that is its designated purpose as per:
		http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
]]

QTMacros = LibStub("AceAddon-3.0"):NewAddon("QTMacros", "AceConsole-3.0", "AceEvent-3.0")
local QT = QTMacros
-- local L = LibStub("AceLocale-3.0"):GetLocale("QTMacros")
local db

local MAX_ACCOUNT_MACROS = 36; -- in case Blizzard_MacroUI isn't loaded


local function _ActionIsMacro(actionID, macroID)
	local actionType, actionInternalID = GetActionInfo(actionID)
	return (actionType == "macro" and actionInternalID == macroID)
end


function QT:BuildMacro(macroText)
	local macroID = GetMacroIndexByName("qt")

	if MacroFrame and MacroFrame:IsShown() then
		MacroFrame_SaveMacro()
	end

	if macroID > 0 then
		EditMacro(macroID, nil, nil, macroText)
	elseif (GetNumMacros()) == MAX_ACCOUNT_MACROS then
		self:Printf("Failed. At limit of %d account macros.", MAX_ACCOUNT_MACROS)
	else
		macroID = CreateMacro("qt", 1, macroText)

		if MacroFrame and MacroFrame.selectedTab == 1 then
			-- The UI is on the account tab, and we just added an account macro; it may impact the selection.
			local selectedMacro = MacroFrame.selectedMacro
			if not selectedMacro then
				-- The tab was empty, but now it's not.
				MacroFrame_SelectMacro(1)
			elseif selectedMacro and selectedMacro >= macroID then
				-- Our new macro bumped the id of the selected macro.
				MacroFrame_SelectMacro(selectedMacro + 1)
			end
		end
	end

	if MacroFrame and MacroFrame:IsShown() then
		MacroFrame_Update();
		MacroPopupFrame:Hide();
		MacroFrameText:ClearFocus();
	end

	return macroID
end


function QT:SelectOurMacro(macroID)
	if not (MacroFrame and MacroFrame:IsShown()) then
		ShowMacroFrame()
	end
	MacroPopupFrame:Hide();
	MacroFrameText:ClearFocus();
	
	if macroID <= MAX_ACCOUNT_MACROS then
		MacroFrameTab1:Click() -- account tab
	else
		MacroFrameTab2:Click() -- character tab
	end
	_G["MacroButton"..(macroID - MacroFrame.macroBase)]:Click()

	-- This won't work until I figure out how to pack containers.
	-- The issue is that the "VerticalScrollRange" isn't determined yet if we're
	-- trying to show the window for the first time since login.

	-- if macroID <= MAX_ACCOUNT_MACROS then
		-- -- The account tab is big; scroll down if it's helpful.
		-- local row = floor((macroID - MacroFrame.macroBase - 1) / NUM_MACROS_PER_ROW)
		-- row = row - 1 -- because it's better to scroll 1 higher, to have the macro in the middle
		-- local scroll = row * 46
		-- self:Print(scroll, MacroButtonScrollFrame:GetVerticalScrollRange())
		-- scroll = max(scroll, 0)
		-- self:Print(scroll, MacroButtonScrollFrame:GetVerticalScrollRange())
		-- scroll = min(scroll, MacroButtonScrollFrame:GetVerticalScrollRange())
		-- self:Print(scroll, MacroButtonScrollFrame:GetVerticalScrollRange())
		-- MacroButtonScrollFrame:SetVerticalScroll(scroll)
	-- end

end


function QT:PlaceMacro(macroID)
	if db.savedActionID and _ActionIsMacro(db.savedActionID, macroID) then
		-- The macro's already on the bar where we want. We're done.
		return
	end
	
	-- The macro isn't on the bar where we expected, but it might be around elsewhere.
	local actionID = nil
	for x=1,120 do
		if _ActionIsMacro(x, macroID) then
			actionID = x
			db.savedActionID = x
			break
		end
	end

	if not actionID then
		if db.savedActionID then
			-- action isn't on the bars, but we can restore it since we know where the user likes it
			PickupMacro(macroID)
			PlaceAction(db.savedActionID)
			ClearCursor()
		else
			-- action isn't on the bars, and we don't know what the user wants. open the macro UI and let him do it himself.
			local macroFrameWasClosed = not (MacroFrame and MacroFrame:IsShown())
			self:SelectOurMacro(macroID)
			self:Print("Place the 'qt' macro on your bars. This choice will be remembered for you.")

			if macroFrameWasClosed then
				-- We opened the frame, so close it when the user's done dragging the macro.
				-- This happens in the ACTIONBAR_SLOT_CHANGED handler unless the user closes it manually first.
				self.tempOpenedMacroUI = true
				if not self.hookedMacroFrame then 
					MacroFrame:HookScript("OnHide", function()
						self.tempOpenedMacroUI = nil
					end)
					self.hookedMacroFrame = true
				end
			end
		end
	end
end


function QT:ACTIONBAR_SLOT_CHANGED(eventName, arg1)
	local actionType, id = GetActionInfo(arg1)
	if actionType == "macro" and (GetMacroInfo(id)) == "qt" then
		if db.savedActionID ~= arg1 then
			self:Print("Macro location updated.")
			db.savedActionID = arg1
			if self.tempOpenedMacroUI then
				HideUIPanel(MacroFrame)
			end
		end
	end
end


function QT:SlashQT(input)
	local macroID = self:BuildMacro("/target " .. input .. "\n")
	if macroID > 0 then
		self:PlaceMacro(macroID)
	end
end


function QT:SlashQTM(input)
	local macroID = GetMacroIndexByName("qt")
	if macroID > 0 then
		self:SelectOurMacro(macroID)
	else
		self:Print("Macro not found. Use /qt to create it.")
	end
end


function QT:OnInitialize()
	self:RegisterChatCommand("qt", "SlashQT")
	self:RegisterChatCommand("qtm", "SlashQTM")

	QTMacrosPerCharDB = QTMacrosPerCharDB or { }
	db = QTMacrosPerCharDB
end


function QT:OnEnable()
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED")
end


function QT:OnDisable()
end
